<?php
// $Id: nd.buildmodes.inc,v 1.1.2.7 2009/09/01 15:37:17 swentel Exp $

/**
 * @file
 * Manage custom build modes.
 */

/**
 * Custom build modes overview.
 */
function nd_custom_build_modes() {
  $output = '';
  $action = arg(4);
  $build_mode = arg(5);
  $custom_build_modes = variable_get('nd_custom_build_modes', array());

  // Delete form.
  if (in_array($build_mode, array_keys($custom_build_modes)) && $action == 'delete') {
    $output .= drupal_get_form('nd_custom_build_mode_delete_form', $build_mode);
  }
  // Edit form.
  elseif (in_array($build_mode, array_keys($custom_build_modes)) && $action == 'edit') {
    $output .= drupal_get_form('nd_custom_build_mode_form', $build_mode);
  }
  // Custom build modes overview form.
  else {
    if (empty($custom_build_modes)) {
      $output .= '<p>'. t('You have not defined any custom build modes.') .'</p>';
    }
    else {
      $header = array(
        t('Name'), t('Key'), t('Operations'),
      );

      $rows = array();
      foreach ($custom_build_modes as $key => $name) {
        $row = array();
        $row[] = check_plain($name);
        $row[] = $key;
        $row[] = l(t('Edit'), 'admin/content/types/buildmodes/edit/'. $key) .' - '. l(t('Delete'), 'admin/content/types/buildmodes/delete/'. $key);
        $rows[] = $row;
      }

      $output .= theme('table', $header, $rows);
    }

    // New content type form.
    $output .= drupal_get_form('nd_custom_build_mode_form');
  }

  return $output;
}

/**
 * Custom display form.
 */
function nd_custom_build_mode_form($form_state, $build_mode = '') {
  $form = array();

  if (!empty($build_mode)) {
    $custom_build_modes = variable_get('nd_custom_build_modes', array());
    $title = $custom_build_modes[$build_mode];
  }
  else {
    $title = '';
  }

  $form['identity'] = array(
    '#type' => 'fieldset',
    '#title' => (empty($build_mode)) ? t('Add new build mode'): t('Update custom build mode'),
    '#collapsible' => (empty($build_mode)) ? TRUE : FALSE,
    '#collapsed' => (empty($build_mode)) ? TRUE : FALSE,
  );
  $form['identity']['key'] = array(
    '#type' => 'textfield',
    '#title' => 'Key name',
    '#description' => t('The machine-readable name of this build mode.'),
    '#required' => TRUE,
  );
  if (!empty($build_mode)) {
    $form['identity']['key']['#disabled'] = TRUE;
    $form['identity']['key']['#value'] = $build_mode;
    $form['identity']['key']['#description'] = t('The machine-readable name of this build mode. Note: you can not edit this field.');
  }
  $form['identity']['name'] = array(
    '#type' => 'textfield',
    '#title' => 'Display name',
    '#description' => t('The name of this build mode which will be used for every content type on the display fields page.'),
    '#required' => TRUE,
    '#default_value' => $title,
  );
  $form['identity']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save custom display'),
  );
  $form['#form_type'] = (empty($build_mode)) ? 'insert' : 'update';

  return $form;
}

/**
 * Validate new custom display submission.
 */
function nd_custom_build_mode_form_validate($form, &$form_state) {
  if ($form['#form_type'] == 'insert') {

    $existing = array_keys(nd_get_build_modes());

    if (in_array($form_state['values']['key'], $existing)) {
      form_set_error('key', t('This display mode already exists.'));
    }

    if (!preg_match('!^[a-z_]+$!', $form_state['values']['key'])) {
      form_set_error('key', t('The machine-readable name must contain only lowercase letters and underscores.'));
    }
  }
}

/**
 * Save new custom build mode.
 */
function nd_custom_build_mode_form_submit($form, &$form_state) {
  $custom_build_modes = variable_get('nd_custom_build_modes', array());
  $custom_build_modes[$form_state['values']['key']] = $form_state['values']['name'];
  variable_set('nd_custom_build_modes', $custom_build_modes);
  drupal_set_message(t('Custom build mode %build_mode has been saved.', array('%build_mode' => $custom_build_modes[$form_state['values']['key']])));
  $form_state['redirect'] = 'admin/content/types/buildmodes';
  menu_rebuild();
}

/**
 * Custom build mode delete form.
 */
function nd_custom_build_mode_delete_form($form_state, $build_mode) {
  $form = array();
  $form['question'] = array(
    '#type' => 'markup',
    '#value' => '<p>'. t('Are you sure you want to delete the build mode type %build_mode ?', array('%build_mode' => $build_mode)) .'</p>',
  );
  $form['display'] = array(
    '#type' => 'value',
    '#value' => $build_mode,
  );
  $form['button']['submit'] = array(
    '#prefix' => '<div>',
    '#type' => 'submit',
    '#value' => t('Delete'),
  );
  $form['buttons']['cancel'] = array(
    '#suffix' => '</div>',
    '#type' => 'markup',
    '#value' => l('cancel', 'admin/content/types/buildmodes'),
  );
  return $form;
}

/**
 * Delete display.
 */
function nd_custom_build_mode_delete_form_submit($form, &$form_state) {
  $custom_build_modes = variable_get('nd_custom_build_modes', array());
  unset($custom_build_modes[$form_state['values']['display']]);
  variable_set('nd_custom_build_modes', $custom_build_modes);
  drupal_set_message(t('Custom build mode %build_mode has been removed.', array('%build_mode' => $form_state['values']['display'])));
  $form_state['redirect'] = 'admin/content/types/buildmodes';
  menu_rebuild();
}
