<?php
// $Id: nd.core_search.inc,v 1.1.2.3 2009/08/11 20:05:21 zuuperman Exp $

/**
 * @file
 * Node displays search callback override. Uses the nd_search
 * display mode instead of the default one.
 */

/**
 * Menu callback; presents the search form and/or search results.
 */
function nd_core_search_view($type = 'node') {
  // Search form submits with POST but redirects to GET. This way we can keep
  // the search query URL clean as a whistle:
  // search/type/keyword+keyword
  if (!isset($_POST['form_id'])) {
    if ($type == '') {
      // Note: search/node can not be a default tab because it would take on the
      // path of its parent (search). It would prevent remembering keywords when
      // switching tabs. This is why we drupal_goto to it from the parent instead.
      drupal_goto('search/node');
    }

    $keys = search_get_keys();
    // Only perform search if there is non-whitespace search term:
    $results = '';
    if (trim($keys)) {
      // Log the search keys:
      watchdog('search', '%keys (@type).', array('%keys' => $keys, '@type' => module_invoke($type, 'search', 'name')), WATCHDOG_NOTICE, l(t('results'), 'search/'. $type .'/'. $keys));

      // Collect the search results for node. Do not use
      // nd for searching anything else (eg users)
      if ($type == 'node') {
        $results = _nd_node_core_search($keys, $type);
      }
      else {
        $results = search_data($keys, $type);
      }

      if ($results) {
        $results = theme('box', t('Search results'), $results);
      }
      else {
        $results = theme('box', t('Your search yielded no results'), search_help('search#noresults', drupal_help_arg()));
      }
    }

    // Construct the search form.
    $output = drupal_get_form('search_form', NULL, $keys, $type);
    $output .= $results;

    return $output;
  }

  return drupal_get_form('search_form', NULL, empty($keys) ? '' : $keys, $type);
}

/**
 * Helper function to search nodes.
 * We literally copy the function from node_search, case 'search'
 * but with a few changes:
 *
 * - we alter the build mode to 'nd_search'.
 * - instead of returning an array, we directly return the output.
 */
function _nd_node_core_search($keys, $type) {
  // Build matching conditions
  list($join1, $where1) = _db_rewrite_sql();
  $arguments1 = array();
  $conditions1 = 'n.status = 1';

  if ($type = search_query_extract($keys, 'type')) {
    $types = array();
    foreach (explode(',', $type) as $t) {
      $types[] = "n.type = '%s'";
      $arguments1[] = $t;
    }
    $conditions1 .= ' AND ('. implode(' OR ', $types) .')';
    $keys = search_query_insert($keys, 'type');
  }

  if ($category = search_query_extract($keys, 'category')) {
    $categories = array();
    foreach (explode(',', $category) as $c) {
      $categories[] = "tn.tid = %d";
      $arguments1[] = $c;
    }
    $conditions1 .= ' AND ('. implode(' OR ', $categories) .')';
    $join1 .= ' INNER JOIN {term_node} tn ON n.vid = tn.vid';
    $keys = search_query_insert($keys, 'category');
  }

  // Build ranking expression (we try to map each parameter to a
  // uniform distribution in the range 0..1).
  $ranking = array();
  $arguments2 = array();
  $join2 = '';
  // Used to avoid joining on node_comment_statistics twice
  $stats_join = FALSE;
  $total = 0;
  if ($weight = (int)variable_get('node_rank_relevance', 5)) {
    // Average relevance values hover around 0.15
    $ranking[] = '%d * i.relevance';
    $arguments2[] = $weight;
    $total += $weight;
  }
  if ($weight = (int)variable_get('node_rank_recent', 5)) {
    // Exponential decay with half-life of 6 months, starting at last indexed node
    $ranking[] = '%d * POW(2, (GREATEST(MAX(n.created), MAX(n.changed), MAX(c.last_comment_timestamp)) - %d) * 6.43e-8)';
    $arguments2[] = $weight;
    $arguments2[] = (int)variable_get('node_cron_last', 0);
    $join2 .= ' LEFT JOIN {node_comment_statistics} c ON c.nid = i.sid';
    $stats_join = TRUE;
    $total += $weight;
  }
  if (module_exists('comment') && $weight = (int)variable_get('node_rank_comments', 5)) {
    // Inverse law that maps the highest reply count on the site to 1 and 0 to 0.
    $scale = variable_get('node_cron_comments_scale', 0.0);
    $ranking[] = '%d * (2.0 - 2.0 / (1.0 + MAX(c.comment_count) * %f))';
    $arguments2[] = $weight;
    $arguments2[] = $scale;
    if (!$stats_join) {
      $join2 .= ' LEFT JOIN {node_comment_statistics} c ON c.nid = i.sid';
    }
    $total += $weight;
  }
  if (module_exists('statistics') && variable_get('statistics_count_content_views', 0) &&
      $weight = (int)variable_get('node_rank_views', 5)) {
    // Inverse law that maps the highest view count on the site to 1 and 0 to 0.
    $scale = variable_get('node_cron_views_scale', 0.0);
    $ranking[] = '%d * (2.0 - 2.0 / (1.0 + MAX(nc.totalcount) * %f))';
    $arguments2[] = $weight;
    $arguments2[] = $scale;
    $join2 .= ' LEFT JOIN {node_counter} nc ON nc.nid = i.sid';
    $total += $weight;
  }

  // When all search factors are disabled (ie they have a weight of zero),
  // the default score is based only on keyword relevance and there is no need to
  // adjust the score of each item.
  if ($total == 0) {
    $select2 = 'i.relevance AS score';
    $total = 1;
  }
  else {
    $select2 = implode(' + ', $ranking) .' AS score';
  }

  // Do search.
  $find = do_search($keys, 'node', 'INNER JOIN {node} n ON n.nid = i.sid '. $join1, $conditions1 . (empty($where1) ? '' : ' AND '. $where1), $arguments1, $select2, $join2, $arguments2);

  // Load results.
  $nodes = array();
  foreach ($find as $item) {
    // Build the node body.
    $node = node_load($item->sid);
    $node->build_mode = 'nd_search';
    $nodes[] = node_view($node, FALSE, FALSE, FALSE);
  }

  if (!empty($nodes)) {
    $output = theme('item_list', $nodes);
    $output .= theme('pager', NULL, 10, 0);
  }

  return $output;
}
