<?php
// $Id: nd.fields.inc,v 1.1.2.6 2009/09/01 09:39:59 swentel Exp $

/**
 * @file
 * Manage custom fields.
 */

/**
 * Custom fields overview.
 *
 * @param string $node_type The node type.
 */
function nd_custom_fields($node_type) {
  $output = '';
  $action = arg(5);
  $custom_field = arg(6);
  $custom_fields = variable_get('nd_custom_fields', array());

  // Delete form.
  if (in_array($custom_field, array_keys($custom_fields[$node_type])) && $action == 'delete') {
    $output .= drupal_get_form('nd_custom_field_delete_form', $custom_field);
  }
  // Edit form.
  elseif (in_array($custom_field, array_keys($custom_fields[$node_type])) && $action == 'edit') {
    $output .= drupal_get_form('nd_custom_field_form', $node_type, $custom_field);
  }
  // Custom fields overview form
  else {
    if (empty($custom_fields[$node_type])) {
      $output .= '<p>'. t('You have not defined any custom fields.') .'</p>';
    }
    else {
      $header = array(
        t('Label'), t('Key'), t('Operations'),
      );

      $rows = array();
      foreach ($custom_fields[$node_type] as $key => $value) {
        $row = array();
        $row[] = check_plain($value['label']);
        $row[] = $key;
        $row[] = l(t('Edit'), 'admin/content/node-type/'. $node_type .'/customfields/edit/'. $key) .' - '. l(t('Delete'), 'admin/content/node-type/'. $node_type .'/customfields/delete/'. $key);
        $rows[] = $row;
      }

      $output .= theme('table', $header, $rows);
    }

    // New custom field form.
    $output .= drupal_get_form('nd_custom_field_form', $node_type);
  }

  return $output;
}

/**
 * Custom field form.
 *
 * @param string $node_type The node type.
 * @param string $custom_field The custom field.
 */
function nd_custom_field_form($form_state, $node_type, $custom_field = '') {
  $form = array();

  if (!empty($custom_field)) {
    $custom_fields = variable_get('nd_custom_fields', array());
    $field = $custom_fields[$node_type][$custom_field];
  }
  else {
    $field = array('label' => '', 'code' => '');
  }

  $form['identity'] = array(
    '#type' => 'fieldset',
    '#title' => (empty($custom_field)) ? t('Add new custom field'): t('Update custom field'),
    '#collapsible' => (empty($custom_field)) ? TRUE : FALSE,
    '#collapsed' => (empty($custom_field)) ? TRUE : FALSE,
  );
  $form['identity']['key'] = array(
    '#type' => 'textfield',
    '#title' => t('Field title'),
    '#description' => t('The machine-readable name of this field.'),
    '#required' => TRUE,
  );
  if (!empty($custom_field)) {
    $form['identity']['key']['#disabled'] = TRUE;
    $form['identity']['key']['#value'] = $custom_field;
    $form['identity']['key']['#description'] = t('The machine-readable name of this field. Note: you can not edit this field.');
  }
  $form['identity']['label'] = array(
    '#type' => 'textfield',
    '#title' => t('Field label'),
    '#description' => t('The label to use when rendering the output and on the display administration screen.'),
    '#required' => TRUE,
    '#default_value' => $field['label']
  );
  $form['identity']['code'] = array(
    '#type' => 'textarea',
    '#title' => t('Field value'),
    '#default_value' => $field['code']
  );

  _nd_field_node_info($form, $node_type);

  $form['identity']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save custom field'),
  );

  $form['#node_type'] = $node_type;
  $form['#form_type'] = (empty($custom_field)) ? 'insert' : 'update';

  return $form;
}

/**
 * Return info about the value field, including token info.
 *
 * @param array $form The current form.
 * @param string $node_type The node type.
 */
function _nd_field_node_info(&$form, $node_type) {
  $form['identity']['help'] = array(
    '#type' => 'item',
    '#value' => t('Every field will be wrapped in &lt;div class="field-key"&gt;VALUE&lt;/div&gt;. Enter PHP code between &lt;?php ?&gt;. Note that executing incorrect PHP-code can break your Drupal site.'),
  );

  if (module_exists('token')) {
    $form['identity']['help']['tokens'] = array(
      '#type' => 'fieldset',
      '#title' => t('Placeholder tokens'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    $form['identity']['help']['tokens']['info'] = array(
      '#type' => 'item',
      '#value' => theme('token_help', 'node'),
    );
  }
  else {
    $form['identity']['help']['#value'] .= '<br />'. t("To use dynamic placeholder tokens in your custom fields (the ID or title of the current node, for example), download and install the <a href='@token'>Token module</a> from Drupal.org.", array('@token' => 'http://www.drupal.org/project/token'));
  }
}

/**
 * Validate custom field submission, only on insert.
 */
function nd_custom_field_form_validate($form, &$form_state) {
  if ($form['#form_type'] == 'insert') {

    $custom_fields = variable_get('nd_custom_fields', array());
    $existing = array_keys($custom_fields[$form['#node_type']]);

    if (in_array($form_state['values']['key'], $existing)) {
      form_set_error('key', t('This field already exists.'));
    }

    if (!preg_match('!^[a-z_]+$!', $form_state['values']['key'])) {
      form_set_error('key', t('The machine-readable name must contain only lowercase letters and underscores.'));
    }
  }
}

/**
 * Save custom field.
 */
function nd_custom_field_form_submit($form, &$form_state) {
  $node_type = $form['#node_type'];
  $nd_custom_fields = variable_get('nd_custom_fields', array());
  $nd_custom_fields[$node_type][$form_state['values']['key']] = array(
    'label' => $form_state['values']['label'],
    'code' => $form_state['values']['code']
  );
  variable_set('nd_custom_fields', $nd_custom_fields);
  $form_state['redirect'] = 'admin/content/node-type/'. $form['#node_type'] .'/customfields';
  drupal_set_message(t('Custom field %custom_field has been saved.', array('%custom_field' => $form_state['values']['label'])));
}

/**
 * Custom field delete form.
 */
function nd_custom_field_delete_form($form_state, $custom_field) {
  $form = array();
  $form['question'] = array(
    '#type' => 'markup',
    '#value' => '<p>'. t('Are you sure you want to delete the field %custom_field ?', array('%custom_field' => $custom_field)) .'</p>',
  );
  $form['custom_field'] = array(
    '#type' => 'value',
    '#value' => $custom_field,
  );
  $form['node_type'] = array(
    '#type' => 'value',
    '#value' => arg(3),
  );
  $form['button']['submit'] = array(
    '#prefix' => '<div>',
    '#type' => 'submit',
    '#value' => t('Delete'),
  );
  $form['buttons']['cancel'] = array(
    '#suffix' => '</div>',
    '#type' => 'markup',
    '#value' => l('cancel', 'admin/content/node-type/'. arg(3) .'/customfields'),
  );
  return $form;
}

/**
 * Delete display.
 */
function nd_custom_field_delete_form_submit($form, &$form_state) {
  // @todo remove variables (like weight, region etc)
  $custom_fields = variable_get('nd_custom_fields', array());
  unset($custom_fields[$form_state['values']['node_type']][$form_state['values']['custom_field']]);
  variable_set('nd_custom_fields', $custom_fields);
  drupal_set_message(t('Custom field %custom_field has been removed.', array('%custom_field' => $form_state['values']['custom_field'])));
  $form_state['redirect'] = 'admin/content/node-type/'. $form_state['values']['node_type'] .'/customfields';
}
